#!/usr/bin/env python3
# Copyright (C) <2018-2024>  <Agence Data Services, DSI France Travail>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.


"""Main router of the REST API"""


from fastapi import APIRouter
from . import functional, technical

# Create the main router
main_routeur = APIRouter()

# Add the technical and functional routers
main_routeur.include_router(functional.router, tags=["functional"])
main_routeur.include_router(technical.router, tags=["technical"])

__all__ = ["main_routeur"]


#!/usr/bin/env python3
# Copyright (C) <2018-2024>  <Agence Data Services, DSI France Travail>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

import uvloop

from prometheus_client import Gauge
from vllm.usage.usage_lib import UsageContext
from vllm import AsyncEngineArgs
from vllm.entrypoints.openai.rpc.server import AsyncEngineRPCServer, run_server


def run_rpc_server(async_engine_args: AsyncEngineArgs,
                   usage_context: UsageContext, rpc_path: str):
    server = AsyncEngineRPCServer(async_engine_args=async_engine_args, usage_context=usage_context, rpc_path=rpc_path)
    model_consumed_memory = Gauge("model_memory_usage", "Model Consumed GPU Memory in GB ")
    model_consumed_memory.set(round(server.engine.engine.model_executor.driver_worker.model_runner.model_memory_usage/float(2**30),2)) # type: ignore
    uvloop.run(run_server(server))
     


#!/usr/bin/env python3
# Copyright (C) <2018-2024>  <Agence Data Services, DSI France Travail>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
import asyncio
import uvicorn
import argparse

from vllm.entrypoints.launcher import serve_http 
import vllm.entrypoints.openai.api_server as vllm_api_server


from happy_vllm.utils_args import parse_args
from happy_vllm.rpc.server import run_rpc_server
from happy_vllm.application import declare_application


TIMEOUT_KEEP_ALIVE = 5 # seconds

def main(**uvicorn_kwargs) -> None:
    args = parse_args()
    asyncio.run(launch_app(args, **uvicorn_kwargs))
    

def happy_vllm_build_async_engine_client(args):
    """Replace vllm.entrypoints.openai.api_server.run_rpc_server by happy_vllm.run_rpc_server
    """
    vllm_api_server.run_rpc_server  = run_rpc_server
    return vllm_api_server.build_async_engine_client(args)


async def launch_app(args, **uvicorn_kwargs):
    async with happy_vllm_build_async_engine_client(args) as async_engine_client:
        app = await declare_application(async_engine_client, args=args)
        shutdown_task = await serve_http(app,
                                        engine=async_engine_client,
                                        host=args.host,
                                        port=args.port,
                                        log_level=args.uvicorn_log_level,
                                        timeout_keep_alive=TIMEOUT_KEEP_ALIVE,
                                        ssl_keyfile=args.ssl_keyfile,
                                        ssl_certfile=args.ssl_certfile,
                                        ssl_ca_certs=args.ssl_ca_certs,
                                        ssl_cert_reqs=args.ssl_cert_reqs,
                                        **uvicorn_kwargs)
    await shutdown_task

if __name__ == "__main__":
    main()